package com.atguigu.ssyx.search.service.impl;

import com.atguigu.ssyx.client.ActivityFeignClient;
import com.atguigu.ssyx.client.product.ProductFeignClient;
import com.atguigu.ssyx.common.context.BaseContext;
import com.atguigu.ssyx.enums.SkuType;
import com.atguigu.ssyx.model.product.Category;
import com.atguigu.ssyx.model.product.SkuInfo;
import com.atguigu.ssyx.model.search.SkuEs;
import com.atguigu.ssyx.search.repository.SkuRepository;
import com.atguigu.ssyx.search.service.SkuService;
import com.atguigu.ssyx.vo.search.SkuEsQueryVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class SkuServiceImpl implements SkuService {
    @Autowired
    private SkuRepository skuRepository;
    @Autowired
    private ProductFeignClient productFeignClient;
    @Autowired
    private ActivityFeignClient activityFeignClient;
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    /**
     * 上架商品
     *
     * @param skuId 商品ID
     */
    @Override
    public void upperSku(Long skuId) {
        SkuInfo skuInfo = productFeignClient.getSkuInfo(skuId).getData();
        Category category = productFeignClient.getCategory(skuId).getData();
        // 获取数据封装SkuEs
        SkuEs skuEs = new SkuEs();
        if (category != null) {
            skuEs.setCategoryId(category.getId());
            skuEs.setCategoryName(category.getName());
        }
        if (skuInfo != null) {
            skuEs.setId(skuInfo.getId());
            skuEs.setKeyword(skuInfo.getSkuName() + "," + skuEs.getCategoryName());
            skuEs.setWareId(skuInfo.getWareId());
            skuEs.setIsNewPerson(skuInfo.getIsNewPerson());
            skuEs.setImgUrl(skuInfo.getImgUrl());
            skuEs.setTitle(skuInfo.getSkuName());
            if (Objects.equals(skuInfo.getSkuType(), SkuType.COMMON.getCode())) {
                skuEs.setSkuType(0);
                skuEs.setPrice(skuInfo.getPrice().doubleValue());
                skuEs.setStock(skuInfo.getStock());
                skuEs.setSale(skuInfo.getSale());
                skuEs.setPerLimit(skuInfo.getPerLimit());
            }
            skuRepository.save(skuEs);
        }
    }

    /**
     * 下架商品
     *
     * @param skuId 商品ID
     */
    @Override
    public void lowerGoods(Long skuId) {
        skuRepository.deleteById(skuId);
    }

    /**
     * * 获取爆品商品
     *
     * @return 查询的商品
     */
    @Override
    public List<SkuEs> findHotSkuList() {
        Pageable pageable = PageRequest.of(0, 10);
        return skuRepository.findByOrderByHotScoreDesc(pageable).getContent();
    }

    /**
     * * 搜索商品
     *
     * @param pageRequest 分页参数
     * @param vo          分页查询参数
     * @return 分页结果
     */
    @Override
    public Page<SkuEs> search(Pageable pageRequest, SkuEsQueryVo vo) {
        Page<SkuEs> pageModel = null;
        // 想vo中设置wareId当前登录用户的仓库Id
        vo.setWareId(BaseContext.getWareId());
        // 根据SpringData命令规则定义方法，进行条件查询
        String keyword = vo.getKeyword();
        if (StringUtils.isEmpty(keyword)) {
            // 根据分类Id和仓库Id和分页查询
            pageModel = skuRepository.findByCategoryIdAndWareId(vo.getCategoryId(), vo.getWareId(), pageRequest);
        } else {
            // 根据仓库Id和分类Id+keyword查询
            pageModel = skuRepository.findByKeywordAndWareId(vo.getKeyword(), pageRequest);
        }

        List<SkuEs> content = pageModel.getContent();

        if (!CollectionUtils.isEmpty(content)) {
            List<Long> skuIdList = content.stream().map(SkuEs::getId).collect(Collectors.toList());
            Map<Long, List<String>> skuIdRuleListMap = activityFeignClient.findActivity(skuIdList);

            if (skuIdRuleListMap != null) {
                content.forEach(skuEs -> {
                    skuEs.setRuleList(skuIdRuleListMap.get(skuEs.getId()));
                });
            }
        }
        return pageModel;
    }

    /**
     * * 更新商品incrHotScore
     *
     * @param skuId 商品id
     */
    @Override
    public void incrHotScore(Long skuId) {
        // 定义key
        String hotKey = "hotScore";
        // 更新热度，当到达一直值时再去更新ES中内容
        Double hotScore = redisTemplate.opsForZSet().incrementScore(hotKey, "skuId:" + skuId, 1);
        // 规则
        if (hotScore % 10 == 0) {
            Optional<SkuEs> optional = skuRepository.findById(skuId);
            SkuEs skuEs = optional.get();
            skuEs.setHotScore(Math.round(hotScore));
            // 如果有id值是更新，没有id值是添加
            skuRepository.save(skuEs);
        }
    }
}
